/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#include "mork.h"

/******************************************************************************
 * FATAL ERROR
 *****************************************************************************/

void
io_error(void)
{
	uint8_t i;

	led_clear();
	while (true) {
		i = 0;
		while (++i < 10) {
			led_toggle();
			_delay_ms(50);
		}
		_delay_ms(1500);
	}
}

/******************************************************************************
 * BIT BANGING
 *****************************************************************************/

/*
 * Activate PORT
 */
void
io_open(void)
{
	platform_open();
}

/*
 * Deactivate PORT
 */
void
io_close(void)
{
	platform_close();
}

/*
 * Set VPP (PROGRAM OUT)
 */
void
io_set_vpp(uint8_t vpp)
{
	platform_set_vpp(vpp);
}

/*
 * Set PGC (CLOCK OUT)
 */
void
io_set_pgc(uint8_t pgc)
{
	platform_set_pgc(pgc);
}

/*
 * Set PGD (DATA OUT)
 */
void
io_set_pgd(uint8_t pgd)
{
	platform_output();

	platform_set_pgd(pgd);
}

/*
 * Get PGD (DATA IN)
 */
uint8_t
io_get_pgd(void)
{
	platform_input();

	return (platform_get_pgd());
}

/*
 * Clock out bit(s)
 */
void
io_clock_out_bits(uint32_t bits, uint8_t nbits)
{
	platform_output();

	/* Clock out bits 0..N */
	for (uint8_t i = 0; i < nbits; ++i) {
		platform_set_pgd(bits & 1);
		bits >>= 1;
		platform_clock();
	}
}

/*
 * Clock 1 bit in and 2 bits out (2-wire 4-phase)
 */
uint8_t
io_clock_bit_4phase(uint8_t tms, uint8_t tdi)
{
	uint8_t tdo;

	/*********
	 * PHASE 1
	 */
	platform_output();

	platform_set_pgd(tdi);		/* TDI OUTPUT */
	platform_clock();

	/*********
	 * PHASE 2
	 */
	platform_set_pgd(tms);		/* TMS OUTPUT */
	platform_clock();

	/*********
	 * PHASE 3
	 */
	platform_input();

	platform_get_pgd();		/* DISCARD INPUT */
	platform_clock();

	/*********
	 * PHASE 4
	 */
	tdo = platform_get_pgd();	/* TDO INPUT */
	platform_clock();

	return tdo;
}

/*
 * Clock multiple bits in and multiple bits out (2-wire 4-phase)
 */
uint32_t
io_clock_bits_4phase(uint8_t nbits, uint32_t tms, uint32_t tdi)
{
	uint32_t tdo = 0;

	for (uint8_t i = 0; i < nbits; ++i) {
		tdo |= (uint32_t)io_clock_bit_4phase(tms & 1, tdi & 1) << i;
		tms >>= 1;
		tdi >>= 1;
	}
	return tdo;
}
