/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <errno.h>
#include <sysexits.h>
#include <sys/types.h>
#include <libgen.h>

#define BUFLEN (1024)
#define BUFMAX (BUFLEN - 1)

/*******************************************************************************
 *
 * Hex 2 Nibble
 *
 ******************************************************************************/
uint8_t
hex2nib(char c)
{
	uint8_t n = 0;

	if (c >= 'a' && c <= 'f') {
		n = c - 'a' + 10;
	}
	else if (c >= 'A' && c <= 'F') {
		n = c - 'A' + 10;
	}
	else if (c >= '0' && c <= '9') {
		n = c - '0';
	}
	return n;
}

/*******************************************************************************
 *
 * Hex 2 Byte
 *
 ******************************************************************************/
static inline uint8_t
hex2byt(char *s)
{
	return hex2nib(s[0]) << 4 | hex2nib(s[1]);
}

/*******************************************************************************
 *
 * Hex 2 Binary
 *
 ******************************************************************************/
int
hex2bin(FILE *fp, uint16_t *addr, uint8_t *typ, uint8_t *bin, uint8_t *binlen)
{
	char line[BUFLEN];
	int i;
	uint8_t sum, len;

	if (fgets(line, BUFLEN, fp) == NULL)
		return 0;

	if (line[0] != ':')
		return -1;

	line[BUFMAX] = '\0';

	i = strlen(line) - 1;
	while (i >= 0 && (line[i] == '\r' || line[i] == '\n'))
		line[i--] = '\0';
		
	if (i & 1 || i < 10)
		return -1;

	sum = 0;
	*binlen = 0;
	for (i = 1; line[i]; i += 2) {
		bin[*binlen] = hex2byt(&line[i]);
		sum += bin[(*binlen)++];
	}
	if (sum)
		return -1;

	len = bin[0];
	if (*binlen != (len + 5))
		return -1;

	*addr = bin[1] << 8 | bin[2];
	*typ = bin[3];
	
	return len;
}

/*******************************************************************************
 *
 * Hex 2 Header
 *
 ******************************************************************************/
int
hex2header(char *file, char *label)
{
	FILE *fp;
	uint16_t addr, ext_addr = 0;
	uint8_t typ = 0, bin[BUFLEN], binlen, *data = &bin[4];
	uint32_t i, address, nbytes = 0;

	fp = fopen(file, "rb");
	if (!fp) {
		return -1;
	}
	printf("const uint8_t %s[] = {\n", label);
	while (typ != 1 && (hex2bin(fp, &addr, &typ, bin, &binlen) > 0)) {
		switch (typ) {
		case 0:	/* data */
			binlen -= 5;
			address = ext_addr << 16 | addr;
			printf("/*[%06X]*/", address);
			for (i = 0; i < binlen; ++i) {
				printf("0x%02X,", data[i]);
			}
			nbytes += binlen;
			putchar('\n');
			break;
		case 4: /* address */
			ext_addr = bin[4] << 8 | bin[5];
			break;
		}
	}
	printf("}; /* %d bytes */\n", nbytes);
	fclose(fp);

	return nbytes;
}

/*******************************************************************************
 *
 * HOWTO
 *
 ******************************************************************************/
void
usage(const char *msg, int err)
{
	fprintf(stderr, "USAGE: hex2h HEX-FILE LABEL\n\n");

	if (msg)
		fprintf(stderr, "%s\n\n", msg);

	exit(err);
}

/*******************************************************************************
 *
 * How now brown cow?
 *
 ******************************************************************************/
int
main(int argc, char **argv)
{
	if (argc != 3)
		usage("Missing args.", EX_USAGE);

	if (hex2header(argv[1], argv[2]) < 0)
		usage("Invalid args.", EX_USAGE);
}
