/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#include "mork.h"

/*
 * Initialise
 */

static inline void
initialise(void)
{
	/* Set up system clock and millisecond counter */
	rcc_clock_setup_pll(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_96MHZ]);

	/* clock rate / 1000 to get 1mS interrupt rate */
	systick_set_reload(96000);
	systick_set_clocksource(STK_CSR_CLKSOURCE_AHB);

	/* this done last */
	systick_interrupt_enable();

	/* Enable GPIO clocks. */
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_GPIOD);
	rcc_periph_clock_enable(RCC_OTGFS);

	/* Enable USBOTG */
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO11 | GPIO12);
	gpio_set_af(GPIOA, GPIO_AF10, GPIO11 | GPIO12);
}

/*
 * Advanced LED diagnostics.
 */

void
led_clear(void)
{
	GPIOD_BSRR = (1 << 15);
}

void
led_set(void)
{
	GPIOD_BSRR = (1 << (16 + 15));
}

void
led_toggle(void)
{
	GPIOD_ODR ^= (1 << 15);
}

static inline void
led_init(void)
{
	gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO15);
	led_clear();
}

/*
 * Program PICMicro.
 */
int
main(void)
{
	/* MCU init */
	initialise();

	/* LED init */
	led_init();

	/* UART init */
	uart_init();

	while (true) {
		/* Program Chip */
		io_open();
		stk500v2_listen();
		io_close();
	}

	return (0);
}
