/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#ifndef _STM32F4D_H
#define _STM32F4D_H

#define usb_driver otgfs_usb_driver

/* PA2 = PGC, PA1 = VPP, PA0 = PGD */

#define GPIO_MASK	(0xFFFFFFE0)
#define GPIO_SPEED	(0x0000002A)	/* 50 MHz */
#define GPIO_OUT	(0x00000015)	/* 0=O/P 1/2=O/P */
#define GPIO_IN		(0x00000014)	/* 0=I/P 1/2=O/P */

static inline void
platform_output(void)
{
	GPIOA_MODER = (GPIOA_MODER & GPIO_MASK) | GPIO_OUT;
}

static inline void
platform_input(void)
{
	GPIOA_MODER = (GPIOA_MODER & GPIO_MASK) | GPIO_IN;
}

static inline void
platform_set_vpp(uint8_t vpp)
{
	if (vpp)
		GPIOA_BSRR = (1 << 1);
	else
		GPIOA_BSRR = (1 << 17);
}

static inline void
platform_set_pgc(uint8_t pgc)
{
	if (pgc)
		GPIOA_BSRR = (1 << 2);
	else
		GPIOA_BSRR = (1 << 18);
}

static inline void
platform_set_pgd(uint8_t pgd)
{
	if (pgd)
		GPIOA_BSRR = (1 << 0);
	else
		GPIOA_BSRR = (1 << 16);
}

static inline uint8_t
platform_get_pgd(void)
{
	return (GPIOA_IDR & 1);
}

static inline void
platform_clock(void)
{
	GPIOA_BSRR = (1 << 2);
	for (uint8_t i = 0; i < 7; ++i)
		__asm__("nop");

	GPIOA_BSRR = (1 << 18);
	for (uint8_t i = 0; i < 7; ++i)
		__asm__("nop");
}

static inline void
platform_open(void)
{
	platform_set_vpp(HIGH);	/* RUN */
	platform_output();
	GPIOA_OSPEEDR = (GPIOA_OSPEEDR & GPIO_MASK) | GPIO_SPEED;
}

static inline void
platform_close(void)
{
	GPIOA_MODER &= GPIO_MASK;
	GPIOA_OSPEEDR &= GPIO_MASK;
}

void led_clear(void);
void led_set(void);
void led_toggle(void);
int main(void);

#endif /* !_STM32F4D_H */
