/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#include "mork.h"

/* VCP Input buffer */
extern uint8_t inbuf[CDCACM_PKTLEN_IN];
extern uint8_t inptr, inlen;

/* VCP Output buffer */
extern uint8_t outbuf[CDCACM_PKTLEN_OUT];
extern uint8_t outlen;

void
uart_init(void)
{
	cdcacm_init();
}

void
uart_shutdown(void)
{
	_delay_ms(1);
}

/*
 * Poll input buffer
 *
 * Return true if data available else false when none
 */
int16_t
uart_poll(void)
{
	if (inptr != inlen)
		return true;

	inptr = 0;
	inlen = 0;
	cdcacm_poll();

	return (inptr != inlen);
}

/*
 * Return character on success or -1 on FAILURE (NOT IMPLEMENTED)
 */
int16_t
uart_getchar(void)
{
	static uint16_t i = 0;

	while (uart_poll() == false) {
		if (!i++)
			led_toggle();
	}
	if (i++ & 0x2000)
		led_set();
	else
		led_clear();

	return (inbuf[inptr++]);
}

/*
 * Flush output buffer
 */
void
uart_flush(void)
{
	if (outlen) {
		while (cdcacm_puts() != outlen)
			;
		outlen = 0;
	}
}

/*
 * Return 0 on success or -1 on FAILURE (NOT IMPLEMENTED)
 */
int16_t
uart_putchar(char c)
{
	outbuf[outlen++] = c;

	if (outlen == CDCACM_PKTLEN_OUT)
		uart_flush();

	return (0);
}
