/*
 * Copyright (C) 2014-2015 Darron Broad
 * All rights reserved.
 *
 * This file is part of Mork Microchip PIC ICSP.
 *
 * Mork Microchip PIC ICSP is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * Mork Microchip PIC ICSP is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Mork Microchip PIC ICSP. If not, see http://www.gnu.org/licenses/
 */

#include "mork.h"

/*
 * Initialise
 */

static inline void
initialise(void)
{
	/* Set up system clock and millisecond counter */
	rcc_clock_setup_pll(&rcc_hse_configs[RCC_CLOCK_HSE8_72MHZ]);

	/* 72MHz / 8 => 9000000 counts per second */
	systick_set_clocksource(STK_CSR_CLKSOURCE_AHB_DIV8);

	/* 9000000/9000 = 1000 overflows per second - every 1ms one interrupt */
	/* SysTick interrupt every N clock pulses: set reload to N-1 */
	systick_set_reload(8999);

	systick_interrupt_enable();

	/* Enable GPIO clocks. */
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_GPIOC);
}

/*
 * Advanced LED diagnostics.
 */

void
led_clear(void)
{
	GPIOC_BSRR = (1 << 13);
}

void
led_set(void)
{
	GPIOC_BRR = (1 << 13);
}

void
led_toggle(void)
{
	GPIOC_ODR ^= (1 << 13);
}

static inline void
led_init(void)
{
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_50_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO13);
	led_clear();
}

/*
 * Program PICMicro.
 */
int
main(void)
{
	/* MCU init */
	initialise();

	/* LED init */
	led_init();

	/* UART init */
	uart_init();

	while (true) {
		/* Program Chip */
		io_open();
		stk500v2_listen();
		io_close();
	}

	return (0);
}
